(ns rlm.map-utils
    "A lot of times you want to transform maps by some function
     that operates only on their keys or values, but then get a
     map object back again.  This makes that sort of thing easier."
  ^{:author "Robert McIntyre"}
  (:use [rlm
	 [dreams :only [defn-decorated]]
	 [decorators :only [preserve-meta]]]))


(defn map-keys
  "Transform a map by applying a function to its keys
   and \"dragging along\" its values.  the transformation
   function must be one-to-one for the keys of the map
   and its range or else the resulting map would have more than
   one value for a key"
  [f m]
  (zipmap (map f (keys m)) (vals m)))

(defn map-vals
  "Transform a map by applying a function to its values, keeping the
   keys the same."
  [f m]
  (zipmap (keys m) (map f (vals m))))


(defn filter-keys
  "Makes a new map which contains only pairs for which
   the filter function on the key of the pair returns true."
  [fun m]
  (select-keys m (filter fun (keys m))))


(defn filter-vals
  "Makes a new map which contains only pairs for which
   the filter function on the value of the pair returns true."
  [fun m]
    (into {} (filter (comp fun val) m)))


(defn sort-map-by
  "returns a new sorted map using the provided function as a comparator
   the comparator function takes two KEYS and compares them"
  [fn map]
  (into (sorted-map-by fn ) map))

(defn def-map
  "takes a map and creates in the current namespace symbols
  corresponding to its keys with values corresponding to its
  values. If the symbols already exist, it just overwrites them.
  Great for debugging at the REPL because you can just define a test
  map, then establish all the key-val variables, then copy snippets
  from a function which references a map in this way and directly test
  the code with those values."
  [])
