(ns rlm.vision
  (:use [clojure.contrib def trace])
  (:use [rlm image-utils])
  (:use [cantor])
  (:import java.awt.image.BufferedImage
	   	   [ij ImagePlus IJ]

	   ))

(defn select-last [& args]
  (last args))
  

;this is to finally get these stupid color problems out of the way!

(defmulti gen-color (fn [& args]	
	      (class (last args))))




(defmethod gen-color (class 5)
  ([r g b]
     (gen-color (vector r g b 255)))
  ([r g b a]
     (gen-color (vector r g b a))))

(defmethod gen-color (class :black)
  [key]
  (get {:black (gen-color {:r 0 :g 0 :b 0})
	:white (gen-color {:r 255 :g 255 :b 255})}
       key
       (gen-color {:r 0 :g 0 :b 0})))

(defmethod gen-color (class [5 10 255 50])
  [[r g b a]]
  (gen-color {:r r :g g :b b :a a}))

(defmethod gen-color (class {:r 5 :g 10 :b 255 :a 50})
  [{:keys [r g b a] :or {r 0 g 0 b 0 a 255}}]
  (let [r (int (rem r 256))
	g (int (rem g 256))
	b (int (rem b 256))
	a (int (rem a 256))
	]
    (+
     (bit-shift-left a 24)
     (bit-shift-left r 16)
     (bit-shift-left g  8)
     b)))

(def color (memoize gen-color))

  


(defn black-lattice [x y]
  (if (and
      (< (rem y 500) 10)
      (< (rem x 500) 10)) (color :black) (color :white)))



(defn snake-problem-2 [x y]
  (let [E (+ 4 (* 3 (java.lang.Math/cos x)) (java.lang.Math/cos (* 3 x)))
	E (int (* E 255/9))]
    
    (color E E E)))



(defn function->image [[xmin xmax] [ymin ymax] f] 
  (let [width (- xmax xmin)
	height (- ymax ymin)
	image (BufferedImage. width height BufferedImage/TYPE_INT_RGB)]

    (doall 
    (for [y (range ymin ymax) x (range xmin xmax)]
      (.setRGB image (- x xmin) (- y ymin) (f x y))))

    (ImagePlus. "functionally-generated" image)))

(defn better-function->image [[xmin xmax] [ymin ymax] f] 
  
  (let [width (- xmax xmin)
	height (- ymax ymin)
	image (BufferedImage. width height BufferedImage/TYPE_INT_RGB)
	set-color (fn [[x y]] (.setRGB image (- x xmin) (- y ymin) (f x y)))
	domain (for [x (range xmin xmax) y (range ymin ymax)] [x y])]
	
    (dorun (pmap set-color domain)) 
    (ImagePlus. "functionally-generated" image)))





(def a-face (normalize (cross
	     (vec3 1 0 0)
	     (sub
	      (vec3 0 0 (/ (* 2 (expt 3 1/2))))
	      (vec3 (/ (- (sqrt 3)) 2) (- 1/2) 0)))))

(def c-face (mul  (normalize (cross
	     (vec3 (/ (sqrt 3) 2) 3/2 0)
	     (sub
	      (vec3 0 0 (/ (* 2 (expt 3 1/2))))
	      (vec3 (/ (- (sqrt 3)) 2)    (- 1/2)  0)))) -1))


(def b-face (mul (normalize
	     (cross
	      (vec3 (/ (sqrt 3) 2) (- 3/2) 0)
	      (sub
	       (vec3 0 0 (/ (* 2 (expt 3 1/2))))
	       (vec3 (/ (sqrt 3) 2) (- 1/2) 0)))) -1))
